@@ -292,6 +292,15 @@ public:
 		}
 		return 0;
 	}
+
+	int suspended()
+	{
+		if(isAttached)
+		{
+			return 0;
+		}
+		return server.suspended();
+	}
 };
 
 class Receiver :	public ByteSoundReceiver_skel,
@@ -588,6 +597,12 @@ public:
 		return ARTS_E_NOSERVER;
 	}
 
+	int suspended() {
+		if(!server.isNull()) 
+			return server.suspended()? 1:0;
+		return ARTS_E_NOSERVER;
+	}
+
 	void free() {
 		// nothing to do
 	}
@@ -710,6 +725,13 @@ extern "C" int arts_backend_suspend()
 	return ArtsCApi::the()->suspend();	
 }
 
+extern "C" int arts_backend_suspended()
+{
+	if(!ArtsCApi::the()) return ARTS_E_NOINIT;
+	arts_backend_debug("arts_backend_suspended");
+	return ArtsCApi::the()->suspended();	
+}
+
 extern "C" void arts_backend_free()
 {
 	if(!ArtsCApi::the()) return;
