@@ -200,7 +200,16 @@ int open (const char *pathname, int flags, ...)
    * will work.
    */
   va_start(args,flags);
-  if(flags & O_CREAT) mode = va_arg(args, mode_t);
+  if(flags & O_CREAT) {
+	/* The compiler will select one of these at compile-tyime if -O is used.
+	 * Otherwise, it may be deferred until runtime.
+	 */
+    if (sizeof(int) >= sizeof(mode_t)) {
+      mode = va_arg(args, int);
+    } else {
+      mode = va_arg(args, mode_t);
+    }
+  }
   va_end(args);
 
   if (!is_sound_device(pathname))    /* original open for anything but sound */
