@@ -211,24 +211,26 @@ gsl_g_convert (const gchar  *str,
 #define g_snprintf gsl_g_snprintf
 static void
 gsl_g_snprintf (gchar        *out_buffer,
-                size_t        size,
-                const gchar  *format,
+		size_t        size,
+		const gchar  *format,
 		...)
 {
   gchar *buffer;
   va_list args;
-
+  
   /* print string into large enough buffer */
   va_start (args, format);
   buffer = g_strdup_printf (format, args);
   va_end (args);
-
+  
   /* copy the first size bytes into out_buffer */
   strncpy (out_buffer, buffer, size);
+  out_buffer[ size - 1 ] = '\0';
   g_free (buffer);
 }
 #endif
 
+
 struct _GString
 {
   gchar *str;
@@ -236,7 +238,7 @@ struct _GString
 };
 
 static GString*
-g_string_new(char*s)
+g_string_new(const gchar* s)
 {
   GString *gstr=g_new0(GString,1);
   gstr->str= g_strdup(s);
@@ -1080,7 +1082,7 @@ g_scanner_unexp_token (GScanner		*scanner,
   guint	token_string_len;
   gchar	*expected_string;
   guint	expected_string_len;
-  gchar	*message_prefix;
+  const gchar *message_prefix;
   gboolean print_unexp;
   void (*msg_handler)	(GScanner*, const gchar*, ...);
   
@@ -1928,8 +1930,8 @@ g_scanner_get_token_ll	(GScanner	*scanner,
 	  config->scan_identifier_NULL &&
 	  strlen (value.v_identifier) == 4)
 	{
-	  gchar *null_upper = "NULL";
-	  gchar *null_lower = "null";
+	  const gchar *null_upper = "NULL";
+	  const gchar *null_lower = "null";
 	  
 	  if (scanner->config->case_sensitive)
 	    {
