@@ -16,7 +16,6 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#define	__GSL_MASTER_C__
 #include "gslopmaster.h"
 
 #include "gslcommon.h"
@@ -31,28 +30,7 @@
 
 
 
-/* force public constantness but allow us to change values
- * in these structures.
- * *** sync this with gslengine.h ***
- */
-struct _GslJStream
-{
-  const gfloat      **values;
-  /* const */ guint   n_connections;
-  guint               user_flags : 16;
-};
-struct _GslIStream
-{
-  const gfloat     *values;
-  guint             user_flags : 16;
-  /* const */ guint connected : 1;
-};
-struct _GslOStream
-{
-  gfloat           *values;
-  guint             user_flags : 16;
-  /* const */ guint connected : 1;
-};
+#define	NODE_FLAG_RECONNECT(node)	G_STMT_START { (node)->reconnected = (node)->module.klass->reconnect != NULL; } G_STMT_END
 
 
 /* --- time stamping (debugging) --- */
@@ -79,7 +57,7 @@ struct _Poll
   GslPollFunc poll_func;
   gpointer    data;
   guint       n_fds;
-  GslPollFD  *fds;
+  GPollFD    *fds;
   GslFreeFunc free_func;
 };
 
@@ -89,33 +67,33 @@ static void	master_schedule_discard	(void);
 
 
 /* --- variables --- */
-static gboolean	    master_need_reflow = FALSE;
-static gboolean	    master_need_process = FALSE;
-static OpNode	   *master_consumer_list = NULL;
-const gfloat        gsl_engine_master_zero_block[GSL_STREAM_MAX_VALUES] = { 0, }; /* FIXME */
-static Poll	   *master_poll_list = NULL;
-static guint        master_n_pollfds = 0;
-static guint        master_pollfds_changed = FALSE;
-static GslPollFD    master_pollfds[GSL_ENGINE_MAX_POLLFDS];
-static OpSchedule  *master_schedule = NULL;
+static gboolean	       master_need_reflow = FALSE;
+static gboolean	       master_need_process = FALSE;
+static EngineNode     *master_consumer_list = NULL;
+const gfloat           gsl_engine_master_zero_block[GSL_STREAM_MAX_VALUES] = { 0, }; /* FIXME */
+static Poll	      *master_poll_list = NULL;
+static guint           master_n_pollfds = 0;
+static guint           master_pollfds_changed = FALSE;
+static GPollFD         master_pollfds[GSL_ENGINE_MAX_POLLFDS];
+static EngineSchedule *master_schedule = NULL;
 
 
 /* --- functions --- */
 static void
-add_consumer (OpNode *node)
+add_consumer (EngineNode *node)
 {
-  g_return_if_fail (OP_NODE_IS_CONSUMER (node) && node->toplevel_next == NULL && node->integrated);
-
+  g_return_if_fail (ENGINE_NODE_IS_CONSUMER (node) && node->toplevel_next == NULL && node->integrated);
+  
   node->toplevel_next = master_consumer_list;
   master_consumer_list = node;
 }
 
 static void
-remove_consumer (OpNode *node)
+remove_consumer (EngineNode *node)
 {
-  OpNode *tmp, *last = NULL;
-
-  g_return_if_fail (!OP_NODE_IS_CONSUMER (node) || !node->integrated);
+  EngineNode *tmp, *last = NULL;
+  
+  g_return_if_fail (!ENGINE_NODE_IS_CONSUMER (node) || !node->integrated);
   
   for (tmp = master_consumer_list; tmp; last = tmp, tmp = last->toplevel_next)
     if (tmp == node)
@@ -129,37 +107,70 @@ remove_consumer (OpNode *node)
 }
 
 static void
-op_node_disconnect (OpNode *node,
-		    guint   istream)
+master_idisconnect_node (EngineNode *node,
+			 guint       istream)
 {
-  OpNode *src_node = node->inputs[istream].src_node;
+  EngineNode *src_node = node->inputs[istream].src_node;
   guint ostream = node->inputs[istream].src_stream;
   gboolean was_consumer;
-
-  g_assert (ostream < OP_NODE_N_OSTREAMS (src_node) &&
+  
+  g_assert (ostream < ENGINE_NODE_N_OSTREAMS (src_node) &&
 	    src_node->outputs[ostream].n_outputs > 0);	/* these checks better pass */
-
+  
   node->inputs[istream].src_node = NULL;
   node->inputs[istream].src_stream = ~0;
   node->module.istreams[istream].connected = FALSE;
-  was_consumer = OP_NODE_IS_CONSUMER (src_node);
+  was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
   src_node->outputs[ostream].n_outputs -= 1;
   src_node->module.ostreams[ostream].connected = src_node->outputs[ostream].n_outputs > 0;
   src_node->output_nodes = gsl_ring_remove (src_node->output_nodes, node);
+  NODE_FLAG_RECONNECT (node);
+  NODE_FLAG_RECONNECT (src_node);
   /* add to consumer list */
-  if (!was_consumer && OP_NODE_IS_CONSUMER (src_node))
+  if (!was_consumer && ENGINE_NODE_IS_CONSUMER (src_node))
     add_consumer (src_node);
 }
 
 static void
-op_node_disconnect_outputs (OpNode *src_node,
-			    OpNode *dest_node)
+master_jdisconnect_node (EngineNode *node,
+			 guint       jstream,
+			 guint       con)
 {
-  guint i;
+  EngineNode *src_node = node->jinputs[jstream][con].src_node;
+  guint i, ostream = node->jinputs[jstream][con].src_stream;
+  gboolean was_consumer;
+  
+  g_assert (ostream < ENGINE_NODE_N_OSTREAMS (src_node) &&
+	    node->module.jstreams[jstream].n_connections > 0 &&
+	    src_node->outputs[ostream].n_outputs > 0);	/* these checks better pass */
+  
+  i = --node->module.jstreams[jstream].n_connections;
+  node->jinputs[jstream][con] = node->jinputs[jstream][i];
+  node->module.jstreams[jstream].values[i] = NULL; /* float**values 0-termination */
+  was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
+  src_node->outputs[ostream].n_outputs -= 1;
+  src_node->module.ostreams[ostream].connected = src_node->outputs[ostream].n_outputs > 0;
+  src_node->output_nodes = gsl_ring_remove (src_node->output_nodes, node);
+  NODE_FLAG_RECONNECT (node);
+  NODE_FLAG_RECONNECT (src_node);
+  /* add to consumer list */
+  if (!was_consumer && ENGINE_NODE_IS_CONSUMER (src_node))
+    add_consumer (src_node);
+}
 
-  for (i = 0; i < OP_NODE_N_ISTREAMS (dest_node); i++)
+static void
+master_disconnect_node_outputs (EngineNode *src_node,
+				EngineNode *dest_node)
+{
+  gint i, j;
+  
+  for (i = 0; i < ENGINE_NODE_N_ISTREAMS (dest_node); i++)
     if (dest_node->inputs[i].src_node == src_node)
-      op_node_disconnect (dest_node, i);
+      master_idisconnect_node (dest_node, i);
+  for (j = 0; j < ENGINE_NODE_N_JSTREAMS (dest_node); j++)
+    for (i = 0; i < dest_node->module.jstreams[j].n_connections; i++)
+      if (dest_node->jinputs[j][i].src_node == src_node)
+	master_jdisconnect_node (dest_node, j, i--);
 }
 
 static void
@@ -167,67 +178,71 @@ master_process_job (GslJob *job)
 {
   switch (job->job_id)
     {
-      OpNode *node, *src_node;
+      EngineNode *node, *src_node;
       Poll *poll, *poll_last;
-      guint istream, ostream;
-      GslFlowJob *fjob;
+      guint istream, jstream, ostream, con;
+      EngineFlowJob *fjob;
       gboolean was_consumer;
-    case OP_JOB_INTEGRATE:
+    case ENGINE_JOB_INTEGRATE:
       node = job->data.node;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "integrate(%p)", node);
+      JOB_DEBUG ("integrate(%p)", node);
       g_return_if_fail (node->integrated == FALSE);
       g_return_if_fail (node->sched_tag == FALSE);
-      _gsl_mnl_integrate (node);
-      if (OP_NODE_IS_CONSUMER (node))
+      _engine_mnl_integrate (node);
+      if (ENGINE_NODE_IS_CONSUMER (node))
 	add_consumer (node);
       node->counter = 0;
+      NODE_FLAG_RECONNECT (node);
       master_need_reflow |= TRUE;
       break;
-    case OP_JOB_DISCARD:
+    case ENGINE_JOB_DISCARD:
       /* FIXME: free pending flow jobs */
       node = job->data.node;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "discard(%p)", node);
+      JOB_DEBUG ("discard(%p)", node);
       g_return_if_fail (node->integrated == TRUE);
       /* disconnect inputs */
-      for (istream = 0; istream < OP_NODE_N_ISTREAMS (node); istream++)
+      for (istream = 0; istream < ENGINE_NODE_N_ISTREAMS (node); istream++)
 	if (node->inputs[istream].src_node)
-	  op_node_disconnect (node, istream);
+	  master_idisconnect_node (node, istream);
+      for (jstream = 0; jstream < ENGINE_NODE_N_JSTREAMS (node); jstream++)
+	while (node->module.jstreams[jstream].n_connections)
+	  master_jdisconnect_node (node, jstream, node->module.jstreams[jstream].n_connections - 1);
       /* disconnect outputs */
       while (node->output_nodes)
-	op_node_disconnect_outputs (node, node->output_nodes->data);
+	master_disconnect_node_outputs (node, node->output_nodes->data);
       /* remove from consumer list */
-      if (OP_NODE_IS_CONSUMER (node))
+      if (ENGINE_NODE_IS_CONSUMER (node))
 	{
-	  _gsl_mnl_remove (node);
+	  _engine_mnl_remove (node);
 	  remove_consumer (node);
 	}
       else
-	_gsl_mnl_remove (node);
+	_engine_mnl_remove (node);
       node->counter = 0;
       master_need_reflow |= TRUE;
       master_schedule_discard ();	/* discard schedule so node may be freed */
       break;
-    case GSL_JOB_SET_CONSUMER:
-    case GSL_JOB_UNSET_CONSUMER:
+    case ENGINE_JOB_SET_CONSUMER:
+    case ENGINE_JOB_UNSET_CONSUMER:
       node = job->data.node;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "toggle_consumer(%p)", node);
-      was_consumer = OP_NODE_IS_CONSUMER (node);
-      node->is_consumer = job->job_id == GSL_JOB_SET_CONSUMER;
-      if (was_consumer != OP_NODE_IS_CONSUMER (node))
+      JOB_DEBUG ("toggle_consumer(%p)", node);
+      was_consumer = ENGINE_NODE_IS_CONSUMER (node);
+      node->is_consumer = job->job_id == ENGINE_JOB_SET_CONSUMER;
+      if (was_consumer != ENGINE_NODE_IS_CONSUMER (node))
 	{
-	  if (OP_NODE_IS_CONSUMER (node))
+	  if (ENGINE_NODE_IS_CONSUMER (node))
 	    add_consumer (node);
 	  else
 	    remove_consumer (node);
 	  master_need_reflow |= TRUE;
 	}
       break;
-    case OP_JOB_CONNECT:
+    case ENGINE_JOB_ICONNECT:
       node = job->data.connection.dest_node;
       src_node = job->data.connection.src_node;
-      istream = job->data.connection.dest_istream;
+      istream = job->data.connection.dest_ijstream;
       ostream = job->data.connection.src_ostream;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "connect(%p,%u,%p,%u)", node, istream, src_node, ostream);
+      JOB_DEBUG ("connect(%p,%u,%p,%u)", node, istream, src_node, ostream);
       g_return_if_fail (node->integrated == TRUE);
       g_return_if_fail (src_node->integrated == TRUE);
       g_return_if_fail (node->inputs[istream].src_node == NULL);
@@ -235,44 +250,92 @@ master_process_job (GslJob *job)
       node->inputs[istream].src_stream = ostream;
       node->module.istreams[istream].connected = TRUE;
       /* remove from consumer list */
-      was_consumer = OP_NODE_IS_CONSUMER (src_node);
+      was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
       src_node->outputs[ostream].n_outputs += 1;
       src_node->module.ostreams[ostream].connected = TRUE;
       src_node->output_nodes = gsl_ring_append (src_node->output_nodes, node);
-      src_node->counter = 0;
-      if (was_consumer && !OP_NODE_IS_CONSUMER (src_node))
+      NODE_FLAG_RECONNECT (node);
+      NODE_FLAG_RECONNECT (src_node);
+      src_node->counter = 0;	/* FIXME: counter reset? */
+      if (was_consumer && !ENGINE_NODE_IS_CONSUMER (src_node))
 	remove_consumer (src_node);
       master_need_reflow |= TRUE;
       break;
-    case OP_JOB_DISCONNECT:
+    case ENGINE_JOB_JCONNECT:
       node = job->data.connection.dest_node;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "disconnect(%p,%u)", node, job->data.connection.dest_istream);
+      src_node = job->data.connection.src_node;
+      jstream = job->data.connection.dest_ijstream;
+      ostream = job->data.connection.src_ostream;
+      JOB_DEBUG ("jconnect(%p,%u,%p,%u)", node, jstream, src_node, ostream);
       g_return_if_fail (node->integrated == TRUE);
-      g_return_if_fail (node->inputs[job->data.connection.dest_istream].src_node != NULL);
-      op_node_disconnect (node, job->data.connection.dest_istream);
+      g_return_if_fail (src_node->integrated == TRUE);
+      con = node->module.jstreams[jstream].n_connections++;
+      node->jinputs[jstream] = g_renew (EngineJInput, node->jinputs[jstream], node->module.jstreams[jstream].n_connections);
+      node->module.jstreams[jstream].values = g_renew (const gfloat*, node->module.jstreams[jstream].values, node->module.jstreams[jstream].n_connections + 1);
+      node->module.jstreams[jstream].values[node->module.jstreams[jstream].n_connections] = NULL; /* float**values 0-termination */
+      node->jinputs[jstream][con].src_node = src_node;
+      node->jinputs[jstream][con].src_stream = ostream;
+      /* remove from consumer list */
+      was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
+      src_node->outputs[ostream].n_outputs += 1;
+      src_node->module.ostreams[ostream].connected = TRUE;
+      src_node->output_nodes = gsl_ring_append (src_node->output_nodes, node);
+      NODE_FLAG_RECONNECT (node);
+      NODE_FLAG_RECONNECT (src_node);
+      src_node->counter = 0;	/* FIXME: counter reset? */
+      if (was_consumer && !ENGINE_NODE_IS_CONSUMER (src_node))
+	remove_consumer (src_node);
       master_need_reflow |= TRUE;
       break;
-    case GSL_JOB_ACCESS:
+    case ENGINE_JOB_IDISCONNECT:
+      node = job->data.connection.dest_node;
+      JOB_DEBUG ("idisconnect(%p,%u)", node, job->data.connection.dest_ijstream);
+      g_return_if_fail (node->integrated == TRUE);
+      g_return_if_fail (node->inputs[job->data.connection.dest_ijstream].src_node != NULL);
+      master_idisconnect_node (node, job->data.connection.dest_ijstream);
+      master_need_reflow |= TRUE;
+      break;
+    case ENGINE_JOB_JDISCONNECT:
+      node = job->data.connection.dest_node;
+      jstream = job->data.connection.dest_ijstream;
+      src_node = job->data.connection.src_node;
+      ostream = job->data.connection.src_ostream;
+      JOB_DEBUG ("jdisconnect(%p,%u,%p,%u)", node, jstream, src_node, ostream);
+      g_return_if_fail (node->integrated == TRUE);
+      g_return_if_fail (node->module.jstreams[jstream].n_connections > 0);
+      for (con = 0; con < node->module.jstreams[jstream].n_connections; con++)
+	if (node->jinputs[jstream][con].src_node == src_node &&
+	    node->jinputs[jstream][con].src_stream == ostream)
+	  break;
+      if (con < node->module.jstreams[jstream].n_connections)
+	{
+	  master_jdisconnect_node (node, jstream, con);
+	  master_need_reflow |= TRUE;
+	}
+      else
+	g_warning ("jdisconnect(dest:%p,%u,src:%p,%u): no such connection", node, jstream, src_node, ostream);
+      break;
+    case ENGINE_JOB_ACCESS:
       node = job->data.access.node;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "access node(%p): %p(%p)", node, job->data.access.access_func, job->data.access.data);
+      JOB_DEBUG ("access node(%p): %p(%p)", node, job->data.access.access_func, job->data.access.data);
       g_return_if_fail (node->integrated == TRUE);
       job->data.access.access_func (&node->module, job->data.access.data);
       break;
-    case GSL_JOB_FLOW_JOB:
+    case ENGINE_JOB_FLOW_JOB:
       node = job->data.flow_job.node;
       fjob = job->data.flow_job.fjob;
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "add flow_job(%p,%p)", node, fjob);
+      JOB_DEBUG ("add flow_job(%p,%p)", node, fjob);
       g_return_if_fail (node->integrated == TRUE);
       job->data.flow_job.fjob = NULL;	/* ownership taken over */
-      _gsl_node_insert_flow_job (node, fjob);
-      _gsl_mnl_reorder (node);
+      _engine_node_insert_flow_job (node, fjob);
+      _engine_mnl_reorder (node);
       break;
-    case OP_JOB_DEBUG:
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "debug");
+    case ENGINE_JOB_DEBUG:
+      JOB_DEBUG ("debug");
       g_printerr ("JOB-DEBUG: %s\n", job->data.debug);
       break;
-    case OP_JOB_ADD_POLL:
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "add poll %p(%p,%u)", job->data.poll.poll_func, job->data.poll.data, job->data.poll.n_fds);
+    case ENGINE_JOB_ADD_POLL:
+      JOB_DEBUG ("add poll %p(%p,%u)", job->data.poll.poll_func, job->data.poll.data, job->data.poll.n_fds);
       if (job->data.poll.n_fds + master_n_pollfds > GSL_ENGINE_MAX_POLLFDS)
 	g_error ("adding poll job exceeds maximum number of poll-fds (%u > %u)",
 		 job->data.poll.n_fds + master_n_pollfds, GSL_ENGINE_MAX_POLLFDS);
@@ -290,8 +353,8 @@ master_process_job (GslJob *job)
       poll->next = master_poll_list;
       master_poll_list = poll;
       break;
-    case OP_JOB_REMOVE_POLL:
-      OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "remove poll %p(%p)", job->data.poll.poll_func, job->data.poll.data);
+    case ENGINE_JOB_REMOVE_POLL:
+      JOB_DEBUG ("remove poll %p(%p)", job->data.poll.poll_func, job->data.poll.data);
       for (poll = master_poll_list, poll_last = NULL; poll; poll_last = poll, poll = poll_last->next)
 	if (poll->poll_func == job->data.poll.poll_func && poll->data == job->data.poll.data)
 	  {
@@ -325,7 +388,7 @@ master_process_job (GslJob *job)
     default:
       g_assert_not_reached ();
     }
-  OP_DEBUG (GSL_ENGINE_DEBUG_JOBS, "done");
+  JOB_DEBUG ("done");
 }
 
 static void
@@ -334,7 +397,7 @@ master_poll_check (glong   *timeout_p,
 {
   gboolean need_processing = FALSE;
   Poll *poll;
-
+  
   if (master_need_process || *timeout_p == 0)
     {
       master_need_process = TRUE;
@@ -343,7 +406,7 @@ master_poll_check (glong   *timeout_p,
   for (poll = master_poll_list; poll; poll = poll->next)
     {
       glong timeout = -1;
-
+      
       if (poll->poll_func (poll->data, gsl_engine_block_size (), &timeout,
 			   poll->n_fds, poll->n_fds ? poll->fds : NULL, check_with_revents)
 	  || timeout == 0)
@@ -359,64 +422,79 @@ master_poll_check (glong   *timeout_p,
 }
 
 static inline guint64
-master_handle_flow_jobs (OpNode *node,
-			 guint64 max_tick)
+master_handle_flow_jobs (EngineNode *node,
+			 guint64     max_tick)
 {
-  GslFlowJob *fjob = _gsl_node_pop_flow_job (node, max_tick);
+  EngineFlowJob *fjob = _engine_node_pop_flow_job (node, max_tick);
   
   if_reject (fjob)
     do
       {
-	g_print ("FJob: at:%lld from:%lld \n", node->counter, fjob->any.tick_stamp);
+	g_printerr ("FJob: at:%lld from:%lld \n", node->counter, fjob->any.tick_stamp);
 	switch (fjob->fjob_id)
 	  {
-	  case GSL_FLOW_JOB_ACCESS:
+	  case ENGINE_FLOW_JOB_ACCESS:
 	    fjob->access.access_func (&node->module, fjob->access.data);
 	    break;
 	  default:
 	    g_assert_not_reached (); /* FIXME */
 	  }
-	fjob = _gsl_node_pop_flow_job (node, max_tick);
+	fjob = _engine_node_pop_flow_job (node, max_tick);
       }
     while (fjob);
   
-  return _gsl_node_peek_flow_job_stamp (node);
+  return _engine_node_peek_flow_job_stamp (node);
 }
 
 static void
-master_process_locked_node (OpNode *node,
-			    guint   n_values)
+master_process_locked_node (EngineNode *node,
+			    guint       n_values)
 {
   guint64 final_counter = GSL_TICK_STAMP + n_values;
-
+  
   while (node->counter < final_counter)
     {
       guint64 next_counter = master_handle_flow_jobs (node, node->counter);
       guint64 new_counter = MIN (next_counter, final_counter);
-      guint i, diff = node->counter - GSL_TICK_STAMP;
-
-      for (i = 0; i < OP_NODE_N_ISTREAMS (node); i++)
+      guint i, j, diff = node->counter - GSL_TICK_STAMP;
+      
+      for (i = 0; i < ENGINE_NODE_N_ISTREAMS (node); i++)
 	{
-	  OpNode *inode = node->inputs[i].src_node;
+	  EngineNode *inode = node->inputs[i].src_node;
 	  
 	  if (inode)
 	    {
-	      OP_NODE_LOCK (inode);
+	      ENGINE_NODE_LOCK (inode);
 	      if (inode->counter < final_counter)
 		master_process_locked_node (inode, final_counter - node->counter);
 	      node->module.istreams[i].values = inode->outputs[node->inputs[i].src_stream].buffer;
 	      node->module.istreams[i].values += diff;
-	      OP_NODE_UNLOCK (inode);
+	      ENGINE_NODE_UNLOCK (inode);
 	    }
 	  else
 	    node->module.istreams[i].values = gsl_engine_master_zero_block;
 	}
-      for (i = 0; i < OP_NODE_N_OSTREAMS (node); i++)
+      for (j = 0; j < ENGINE_NODE_N_JSTREAMS (node); j++)
+	for (i = 0; i < node->module.jstreams[j].n_connections; i++)
+	  {
+	    EngineNode *inode = node->jinputs[j][i].src_node;
+	    
+	    ENGINE_NODE_LOCK (inode);
+	    if (inode->counter < final_counter)
+	      master_process_locked_node (inode, final_counter - node->counter);
+	    node->module.jstreams[j].values[i] = inode->outputs[node->jinputs[j][i].src_stream].buffer;
+	    node->module.jstreams[j].values[i] += diff;
+	    ENGINE_NODE_UNLOCK (inode);
+	  }
+      for (i = 0; i < ENGINE_NODE_N_OSTREAMS (node); i++)
+	node->module.ostreams[i].values = node->outputs[i].buffer + diff;
+      if_reject (node->reconnected)
 	{
-	  node->module.ostreams[i].values = node->outputs[i].buffer + diff;
+	  node->module.klass->reconnect (&node->module);
+	  node->reconnected = FALSE;
 	}
       node->module.klass->process (&node->module, new_counter - node->counter);
-      for (i = 0; i < OP_NODE_N_OSTREAMS (node); i++)
+      for (i = 0; i < ENGINE_NODE_N_OSTREAMS (node); i++)
 	{
 	  /* FIXME: this takes the worst possible performance hit to support virtualization */
 	  if (node->module.ostreams[i].values != node->outputs[i].buffer + diff)
@@ -427,93 +505,93 @@ master_process_locked_node (OpNode *node,
     }
 }
 
-static GslLong gsl_trace_delay = 0;
+static GslLong gsl_profile_modules = 0;	/* set to 1 in gdb to get profile output */
 
 static void
 master_process_flow (void)
 {
   guint64 new_counter = GSL_TICK_STAMP + gsl_engine_block_size ();
-  GslLong trace_slowest = 0;
-  GslLong trace_delay = gsl_trace_delay;
-  OpNode *trace_node = NULL;
+  GslLong profile_maxtime = 0;
+  GslLong profile_modules = gsl_profile_modules;
+  EngineNode *profile_node = NULL;
   
   g_return_if_fail (master_need_process == TRUE);
-
+  
   g_assert (gsl_fpu_okround () == TRUE);
-
-  OP_DEBUG (GSL_ENGINE_DEBUG_MASTER, "process_flow");
+  
+  MAS_DEBUG ("process_flow");
   if (master_schedule)
     {
-      OpNode *node;
-
-      _op_schedule_restart (master_schedule);
-      _gsl_com_set_schedule (master_schedule);
+      EngineNode *node;
       
-      node = _gsl_com_pop_unprocessed_node ();
+      _engine_schedule_restart (master_schedule);
+      _engine_set_schedule (master_schedule);
+      
+      node = _engine_pop_unprocessed_node ();
       while (node)
 	{
-	  ToyprofStamp trace_stamp1, trace_stamp2;
-
-	  if_reject (trace_delay)
-	    toyprof_stamp (trace_stamp1);
+	  ToyprofStamp profile_stamp1, profile_stamp2;
+	  
+	  if_reject (profile_modules)
+	    toyprof_stamp (profile_stamp1);
 	  
 	  master_process_locked_node (node, gsl_engine_block_size ());
 	  
-	  if_reject (trace_delay)
+	  if_reject (profile_modules)
 	    {
 	      GslLong duration;
-
-	      toyprof_stamp (trace_stamp2);
-	      duration = toyprof_elapsed (trace_stamp1, trace_stamp2);
-	      if (duration > trace_slowest)
+	      
+	      toyprof_stamp (profile_stamp2);
+	      duration = toyprof_elapsed (profile_stamp1, profile_stamp2);
+	      if (duration > profile_maxtime)
 		{
-		  trace_slowest = duration;
-		  trace_node = node;
+		  profile_maxtime = duration;
+		  profile_node = node;
 		}
 	    }
 	  
-	  _gsl_com_push_processed_node (node);
-	  node = _gsl_com_pop_unprocessed_node ();
+	  _engine_push_processed_node (node);
+	  node = _engine_pop_unprocessed_node ();
 	}
-
-      if_reject (trace_delay)
+      
+      if_reject (profile_modules)
 	{
-	  if (trace_node)
+	  if (profile_node)
 	    {
-	      if (trace_slowest > trace_delay)
+	      if (profile_maxtime > profile_modules)
 		g_print ("Excess Node: %p  Duration: %lu usecs     ((void(*)())%p)         \n",
-			 trace_node, trace_slowest, trace_node->module.klass->process);
+			 profile_node, profile_maxtime, profile_node->module.klass->process);
 	      else
 		g_print ("Slowest Node: %p  Duration: %lu usecs     ((void(*)())%p)         \r",
-			 trace_node, trace_slowest, trace_node->module.klass->process);
+			 profile_node, profile_maxtime, profile_node->module.klass->process);
 	    }
 	}
-
+      
       /* walk unscheduled nodes which have flow jobs */
-      node = _gsl_mnl_head ();
+      node = _engine_mnl_head ();
       while (node && GSL_MNL_HEAD_NODE (node))
 	{
-	  OpNode *tmp = node->mnl_next;
-	  GslFlowJob *fjob = _gsl_node_pop_flow_job (node, new_counter);
-
+	  EngineNode *tmp = node->mnl_next;
+	  EngineFlowJob *fjob = _engine_node_pop_flow_job (node, new_counter);
+	  
 	  if (fjob)
 	    {
 	      while (fjob)
 		{
 		  g_printerr ("ignoring flow_job %p\n", fjob);
-		  fjob = _gsl_node_pop_flow_job (node, new_counter);
+		  fjob = _engine_node_pop_flow_job (node, new_counter);
 		}
-	      _gsl_mnl_reorder (node);
+	      _engine_mnl_reorder (node);
 	    }
 	  node = tmp;
 	}
-
+      
       /* nothing new to process, wait on slaves */
-      _gsl_com_wait_on_unprocessed ();
-
-      _gsl_com_unset_schedule (master_schedule);
+      _engine_wait_on_unprocessed ();
+      
+      _engine_unset_schedule (master_schedule);
       _gsl_tick_stamp_inc ();
-      _gsl_recycle_const_values ();
+      _engine_recycle_const_values ();
     }
   master_need_process = FALSE;
 }
@@ -521,21 +599,21 @@ master_process_flow (void)
 static void
 master_reschedule_flow (void)
 {
-  OpNode *node;
-
+  EngineNode *node;
+  
   g_return_if_fail (master_need_reflow == TRUE);
-
-  OP_DEBUG (GSL_ENGINE_DEBUG_MASTER, "flow_reschedule");
+  
+  MAS_DEBUG ("flow_reschedule");
   if (!master_schedule)
-    master_schedule = _op_schedule_new ();
+    master_schedule = _engine_schedule_new ();
   else
     {
-      _op_schedule_unsecure (master_schedule);
-      _op_schedule_clear (master_schedule);
+      _engine_schedule_unsecure (master_schedule);
+      _engine_schedule_clear (master_schedule);
     }
   for (node = master_consumer_list; node; node = node->toplevel_next)
-    _op_schedule_consumer_node (master_schedule, node);
-  _op_schedule_secure (master_schedule);
+    _engine_schedule_consumer_node (master_schedule, node);
+  _engine_schedule_secure (master_schedule);
   master_need_reflow = FALSE;
 }
 
@@ -543,11 +621,11 @@ static void
 master_schedule_discard (void)
 {
   g_return_if_fail (master_need_reflow == TRUE);
-
+  
   if (master_schedule)
     {
-      _op_schedule_unsecure (master_schedule);
-      _op_schedule_destroy (master_schedule);
+      _engine_schedule_unsecure (master_schedule);
+      _engine_schedule_destroy (master_schedule);
       master_schedule = NULL;
     }
 }
@@ -555,13 +633,13 @@ master_schedule_discard (void)
 
 /* --- MasterThread main loop --- */
 gboolean
-_gsl_master_prepare (GslEngineLoop *loop)
+_engine_master_prepare (GslEngineLoop *loop)
 {
   gboolean need_dispatch;
   guint i;
-
+  
   g_return_val_if_fail (loop != NULL, FALSE);
-
+  
   /* setup and clear pollfds here already, so master_poll_check() gets no junk (and IRIX can't handle non-0 revents) */
   loop->fds_changed = master_pollfds_changed;
   master_pollfds_changed = FALSE;
@@ -570,13 +648,13 @@ _gsl_master_prepare (GslEngineLoop *loop)
   for (i = 0; i < loop->n_fds; i++)
     loop->fds[i].revents = 0;
   loop->revents_filled = FALSE;
-
+  
   loop->timeout = -1;
   /* cached checks first */
   need_dispatch = master_need_reflow || master_need_process;
   /* lengthy query */
   if (!need_dispatch)
-    need_dispatch = op_com_job_pending ();
+    need_dispatch = _engine_job_pending ();
   /* invoke custom poll checks */
   if (!need_dispatch)
     {
@@ -585,59 +663,59 @@ _gsl_master_prepare (GslEngineLoop *loop)
     }
   if (need_dispatch)
     loop->timeout = 0;
-
-  OP_DEBUG (GSL_ENGINE_DEBUG_MASTER, "PREPARE: need_dispatch=%u timeout=%6ld n_fds=%u",
-	    need_dispatch,
-	    loop->timeout, loop->n_fds);
-
+  
+  MAS_DEBUG ("PREPARE: need_dispatch=%u timeout=%6ld n_fds=%u",
+	     need_dispatch,
+	     loop->timeout, loop->n_fds);
+  
   return need_dispatch;
 }
 
 gboolean
-_gsl_master_check (const GslEngineLoop *loop)
+_engine_master_check (const GslEngineLoop *loop)
 {
   gboolean need_dispatch;
-
+  
   g_return_val_if_fail (loop != NULL, FALSE);
   g_return_val_if_fail (loop->n_fds == master_n_pollfds, FALSE);
   g_return_val_if_fail (loop->fds == master_pollfds, FALSE);
   if (loop->n_fds)
     g_return_val_if_fail (loop->revents_filled == TRUE, FALSE);
-
+  
   /* cached checks first */
   need_dispatch = master_need_reflow || master_need_process;
   /* lengthy query */
   if (!need_dispatch)
-    need_dispatch = op_com_job_pending ();
+    need_dispatch = _engine_job_pending ();
   /* invoke custom poll checks */
   if (!need_dispatch)
     {
       glong dummy = -1;
-
+      
       master_poll_check (&dummy, TRUE);
       need_dispatch = master_need_process;
     }
-
-  OP_DEBUG (GSL_ENGINE_DEBUG_MASTER, "CHECK: need_dispatch=%u", need_dispatch);
-
+  
+  MAS_DEBUG ("CHECK: need_dispatch=%u", need_dispatch);
+  
   return need_dispatch;
 }
 
 void
-_gsl_master_dispatch_jobs (void)
+_engine_master_dispatch_jobs (void)
 {
   GslJob *job;
-
-  job = gsl_com_pop_job ();
+  
+  job = _engine_pop_job ();
   while (job)
     {
       master_process_job (job);
-      job = gsl_com_pop_job ();	/* have to process _all_ jobs */
+      job = _engine_pop_job ();	/* have to process _all_ jobs */
     }
 }
 
 void
-_gsl_master_dispatch (void)
+_engine_master_dispatch (void)
 {
   /* processing has prime priority, but we can't process the
    * network, until all jobs have been handled and if necessary
@@ -645,7 +723,7 @@ _gsl_master_dispatch (void)
    * that's why we have to handle everything at once and can't
    * preliminarily return after just handling jobs or rescheduling.
    */
-  _gsl_master_dispatch_jobs ();
+  _engine_master_dispatch_jobs ();
   if (master_need_reflow)
     master_reschedule_flow ();
   if (master_need_process)
@@ -653,50 +731,50 @@ _gsl_master_dispatch (void)
 }
 
 void
-_gsl_master_thread (gpointer data)
+_engine_master_thread (gpointer data)
 {
   gboolean run = TRUE;
-
+  
   /* assert sane configuration checks, since we're simply casting structures */
-  g_assert (sizeof (struct pollfd) == sizeof (GslPollFD) &&
-	    G_STRUCT_OFFSET (GslPollFD, fd) == G_STRUCT_OFFSET (struct pollfd, fd) &&
-	    G_STRUCT_OFFSET (GslPollFD, events) == G_STRUCT_OFFSET (struct pollfd, events) &&
-	    G_STRUCT_OFFSET (GslPollFD, revents) == G_STRUCT_OFFSET (struct pollfd, revents));
-
+  g_assert (sizeof (struct pollfd) == sizeof (GPollFD) &&
+	    G_STRUCT_OFFSET (GPollFD, fd) == G_STRUCT_OFFSET (struct pollfd, fd) &&
+	    G_STRUCT_OFFSET (GPollFD, events) == G_STRUCT_OFFSET (struct pollfd, events) &&
+	    G_STRUCT_OFFSET (GPollFD, revents) == G_STRUCT_OFFSET (struct pollfd, revents));
+  
   /* add the thread wakeup pipe to master pollfds, so we get woken
    * up in time (even though we evaluate the pipe contents later)
    */
   gsl_thread_get_pollfd (master_pollfds);
   master_n_pollfds += 1;
   master_pollfds_changed = TRUE;
-
+  
   toyprof_stampinit ();
   
   while (run)
     {
       GslEngineLoop loop;
       gboolean need_dispatch;
-
-      need_dispatch = _gsl_master_prepare (&loop);
-
+      
+      need_dispatch = _engine_master_prepare (&loop);
+      
       if (!need_dispatch)
 	{
 	  gint err;
-
+	  
 	  err = poll ((struct pollfd*) loop.fds, loop.n_fds, loop.timeout);
 	  
 	  if (err >= 0)
 	    loop.revents_filled = TRUE;
 	  else
 	    g_printerr (G_STRLOC ": poll() error: %s\n", g_strerror (errno));
-
+	  
 	  if (loop.revents_filled)
-	    need_dispatch = _gsl_master_check (&loop);
+	    need_dispatch = _engine_master_check (&loop);
 	}
-
+      
       if (need_dispatch)
-	_gsl_master_dispatch ();
-
+	_engine_master_dispatch ();
+      
       /* handle thread pollfd messages */
       run = gsl_thread_sleep (0);
     }
