@@ -53,18 +53,8 @@ typedef struct {
 } WPos;
 
 
-/* --- prototypes --- */
-static void	wave_chunk_set_loop	(GslWaveChunk   *wchunk,
-					 GslWaveLoopType loop_type,
-					 GslLong         loop_start,
-					 GslLong         loop_end,
-					 guint           loop_count);
-static void	wave_chunk_open		(GslWaveChunk	*wchunk);
-static void	wave_chunk_close	(GslWaveChunk	*wchunk);
-
-
 /* --- variables --- */
-static gfloat static_zero_block[STATIC_ZERO_SIZE] = { 0, };
+static gfloat static_zero_block[STATIC_ZERO_SIZE] = { 0, };	/* FIXME */
 
 
 /* --- functions --- */
@@ -78,25 +68,25 @@ wpos_step (GslWaveChunk *wchunk,
       if (wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG)
 	{
 	  if (wpos->dir < 0 &&
-	      wpos->pos == wchunk->loop_start + wpos->dir)
+	      wpos->pos == wchunk->loop_first + wpos->dir)
 	    {
 	      wpos->loop_count--;
 	      wpos->dir = -wpos->dir;
-	      wpos->pos = wchunk->loop_start + wpos->dir;
+	      wpos->pos = wchunk->loop_first + wpos->dir;
 	    }
-	  else if (wpos->pos == wchunk->loop_end + wpos->dir)
+	  else if (wpos->pos == wchunk->loop_last + wpos->dir)
 	    {
 	      wpos->loop_count--;
 	      wpos->dir = -wpos->dir;
-	      wpos->pos = wchunk->loop_end + wpos->dir;
+	      wpos->pos = wchunk->loop_last + wpos->dir;
 	    }
 	}
       else
 	{
-	  if (wpos->pos == wchunk->loop_end + wpos->dir && wpos->loop_count)
+	  if (wpos->pos == wchunk->loop_last + wpos->dir && wpos->loop_count)
 	    {
 	      wpos->loop_count--;
-	      wpos->pos = wchunk->loop_start;
+	      wpos->pos = wchunk->loop_first;
 	    }
 	}
     }
@@ -110,7 +100,7 @@ fill_block (GslWaveChunk *wchunk,
 	    gboolean	  backward,
 	    guint	  loop_count)
 {
-  GslLong dcache_length = wchunk->dcache->handle->n_values;
+  GslLong dcache_length = gsl_data_handle_length (wchunk->dcache->dhandle);
   guint i, dnode_length = wchunk->dcache->node_size;
   GslDataCacheNode *dnode;
   WPos wpos;
@@ -149,18 +139,18 @@ create_block_for_offset (GslWaveChunk *wchunk,
   GslLong padding = wchunk->n_pad_values;
   GslLong one = wchunk->n_channels;
   GslLong wave_last = wchunk->length - one;
-  GslLong loop_width = wchunk->loop_end - wchunk->loop_start;
+  GslLong loop_width = wchunk->loop_last - wchunk->loop_first;
   gfloat *mem;
   GslLong l, j, k;
 
   if (wchunk->loop_type != GSL_WAVE_LOOP_PINGPONG)
     loop_width += one;
-  
+
   l = length + 2 * padding;
   mem = gsl_new_struct (gfloat, l);
   offset -= padding;
   j = ((wchunk->wave_length - one - offset) -
-       (wchunk->pploop_ends_backwards ? wchunk->loop_start : wave_last - wchunk->loop_end));
+       (wchunk->pploop_ends_backwards ? wchunk->loop_first : wave_last - wchunk->loop_last));
   if (j >= 0)
     {
       k = j / loop_width;
@@ -169,21 +159,21 @@ create_block_for_offset (GslWaveChunk *wchunk,
       if (wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG)
 	{
 	  if (wchunk->pploop_ends_backwards && (k & 1))
-	    fill_block (wchunk, mem, wchunk->loop_end - j, l, FALSE, k);
+	    fill_block (wchunk, mem, wchunk->loop_last - j, l, FALSE, k);
 	  else if (wchunk->pploop_ends_backwards)
-	    fill_block (wchunk, mem, wchunk->loop_start + j, l, TRUE, k);
+	    fill_block (wchunk, mem, wchunk->loop_first + j, l, TRUE, k);
 	  else if (k & 1)
-	    fill_block (wchunk, mem, wchunk->loop_start + j, l, TRUE, k);
+	    fill_block (wchunk, mem, wchunk->loop_first + j, l, TRUE, k);
 	  else
-	    fill_block (wchunk, mem, wchunk->loop_end - j, l, FALSE, k);
+	    fill_block (wchunk, mem, wchunk->loop_last - j, l, FALSE, k);
 	}
       else
-	fill_block (wchunk, mem, wchunk->loop_end - j, l, FALSE, k);
+	fill_block (wchunk, mem, wchunk->loop_last - j, l, FALSE, k);
     }
   else if (wchunk->pploop_ends_backwards)
-    fill_block (wchunk, mem, wchunk->loop_start + j, l, TRUE, 0);
+    fill_block (wchunk, mem, wchunk->loop_first + j, l, TRUE, 0);
   else
-    fill_block (wchunk, mem, wchunk->loop_end - j, l, FALSE, 0);
+    fill_block (wchunk, mem, wchunk->loop_last - j, l, FALSE, 0);
   return mem + padding;
 }
 
@@ -192,7 +182,7 @@ setup_pblocks (GslWaveChunk *wchunk)
 {
   GslLong padding = wchunk->n_pad_values;
   GslLong big_pad = PBLOCK_SIZE (wchunk->n_pad_values, wchunk->n_channels);
-  GslLong loop_width = wchunk->loop_end - wchunk->loop_start;
+  GslLong loop_width = wchunk->loop_last - wchunk->loop_first;
   GslLong one = wchunk->n_channels;
   GslLong loop_duration, wave_last = wchunk->length - one;
   gfloat *mem;
@@ -211,8 +201,8 @@ setup_pblocks (GslWaveChunk *wchunk)
   wchunk->tail.length = wchunk->tail.end - wchunk->tail.start + one;
   if (wchunk->loop_type)
     {
-      wchunk->enter.start = wchunk->loop_end - padding;
-      wchunk->enter.end = wchunk->loop_end + one + big_pad;
+      wchunk->enter.start = wchunk->loop_last - padding;
+      wchunk->enter.end = wchunk->loop_last + one + big_pad;
       wchunk->wrap.start = loop_width - padding;
       wchunk->wrap.end = big_pad;
       if (wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG)
@@ -227,8 +217,8 @@ setup_pblocks (GslWaveChunk *wchunk)
 	}
       else
 	wchunk->wrap.length = loop_width - wchunk->wrap.start + wchunk->wrap.end + one;
-      wchunk->leave_end_norm = wchunk->loop_end + big_pad;
-      wchunk->leave.start = wchunk->loop_end + loop_duration - padding;
+      wchunk->leave_end_norm = wchunk->loop_last + big_pad;
+      wchunk->leave.start = wchunk->loop_last + loop_duration - padding;
       wchunk->leave.end = wchunk->leave_end_norm + loop_duration;
       if (wchunk->mini_loop)
 	{
@@ -239,17 +229,22 @@ setup_pblocks (GslWaveChunk *wchunk)
       wchunk->enter.length = wchunk->enter.end - wchunk->enter.start + one;
       if (wchunk->pploop_ends_backwards)
 	{
-	  wchunk->tail.start += wchunk->loop_end - wave_last + wchunk->loop_start;
-	  wchunk->tail.end += wchunk->loop_end - wave_last + wchunk->loop_start;
+	  wchunk->tail.start += wchunk->loop_last - wave_last + wchunk->loop_first;
+	  wchunk->tail.end += wchunk->loop_last - wave_last + wchunk->loop_first;
 	  wchunk->tail_start_norm = 0 + big_pad;
-	  wchunk->leave_end_norm = wchunk->loop_start - big_pad;
+	  wchunk->leave_end_norm = wchunk->loop_first - big_pad;
 	}
     }
   else
     {
+      /*
       wchunk->enter.start = wchunk->head.end;
       wchunk->enter.end = wchunk->head.end;
       wchunk->enter.length = 0;
+      */
+      wchunk->enter.start = wchunk->tail.start;
+      wchunk->enter.end = wchunk->head.end;
+      wchunk->enter.length = 0;
       wchunk->wrap.start = wchunk->tail.end + 1;
       wchunk->wrap.end = wchunk->head.start - 1;
       wchunk->wrap.length = 0;
@@ -274,14 +269,14 @@ setup_pblocks (GslWaveChunk *wchunk)
       wchunk->enter.mem = mem + padding;
       if (wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG)
 	{
-	  wchunk->wrap.mem = create_block_for_offset (wchunk, wchunk->loop_end + one + wchunk->wrap.start, wchunk->wrap.length);
-	  wchunk->ppwrap.mem = create_block_for_offset (wchunk, wchunk->loop_end + one + wchunk->ppwrap.start, wchunk->ppwrap.length);
+	  wchunk->wrap.mem = create_block_for_offset (wchunk, wchunk->loop_last + one + wchunk->wrap.start, wchunk->wrap.length);
+	  wchunk->ppwrap.mem = create_block_for_offset (wchunk, wchunk->loop_last + one + wchunk->ppwrap.start, wchunk->ppwrap.length);
 	}
       else
 	{
 	  l = wchunk->wrap.length + 2 * padding;
 	  mem = gsl_new_struct (gfloat, l);
-	  fill_block (wchunk, mem, wchunk->loop_start + wchunk->wrap.start - padding, l, FALSE, wchunk->loop_count - 1);
+	  fill_block (wchunk, mem, wchunk->loop_first + wchunk->wrap.start - padding, l, FALSE, wchunk->loop_count - 1);
 	  wchunk->wrap.mem = mem + padding;
 	}
       wchunk->leave.mem = create_block_for_offset (wchunk, wchunk->leave.start, wchunk->leave.length);
@@ -314,15 +309,15 @@ wave_identify_offset (GslWaveChunk *wchunk,
 	g_print ("PHASE_UNDEF, post-tail %ld %ld %ld\n", iter->lbound, iter->rel_pos, iter->ubound);
       return PHASE_UNDEF (wchunk);
     }
-  if (pos <= wchunk->enter.end)					/* before loop */
+  if (pos <= wchunk->head.end)
+    {
+      iter->rel_pos = pos - wchunk->head.start;
+      if (PRINT_DEBUG_INFO)
+	g_print ("PHASE_HEAD %ld %ld %ld\n", wchunk->head.start, iter->rel_pos, wchunk->head.end);
+      return PHASE_HEAD (wchunk);
+    }
+  else if (pos <= wchunk->enter.end)					/* before loop */
     {
-      if (pos <= wchunk->head.end)
-	{
-	  iter->rel_pos = pos - wchunk->head.start;
-	  if (PRINT_DEBUG_INFO)
-	    g_print ("PHASE_HEAD %ld %ld %ld\n", wchunk->head.start, iter->rel_pos, wchunk->head.end);
-	  return PHASE_HEAD (wchunk);
-	}
       if (pos >= wchunk->enter.start)
 	{
 	  iter->rel_pos = pos - wchunk->enter.start;
@@ -337,15 +332,15 @@ wave_identify_offset (GslWaveChunk *wchunk,
 	g_print ("PHASE_NORM, pre-enter %ld %ld %ld\n", iter->lbound, iter->rel_pos, iter->ubound);
       return PHASE_NORM (wchunk);
     }
+  else if (pos >= wchunk->tail.start)
+    {
+      iter->rel_pos = pos - wchunk->tail.start;
+      if (PRINT_DEBUG_INFO)
+	g_print ("PHASE_TAIL %ld %ld %ld\n", wchunk->tail.start, iter->rel_pos, wchunk->tail.end);
+      return PHASE_TAIL (wchunk);
+    }
   else if (pos >= wchunk->leave.start)				/* after loop */
     {
-      if (pos >= wchunk->tail.start)
-	{
-	  iter->rel_pos = pos - wchunk->tail.start;
-	  if (PRINT_DEBUG_INFO)
-	    g_print ("PHASE_TAIL %ld %ld %ld\n", wchunk->tail.start, iter->rel_pos, wchunk->tail.end);
-	  return PHASE_TAIL (wchunk);
-	}
       if (pos <= wchunk->leave.end)
 	{
 	  iter->rel_pos = pos - wchunk->leave.start;
@@ -373,9 +368,9 @@ wave_identify_offset (GslWaveChunk *wchunk,
     }
   else if (wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG)		/* in ping-pong loop */
     {
-      guint loop_width = wchunk->loop_end - wchunk->loop_start;
+      guint loop_width = wchunk->loop_last - wchunk->loop_first;
 
-      pos -= wchunk->loop_end + one;
+      pos -= wchunk->loop_last + one;
       pos %= 2 * loop_width;
       if (pos <= wchunk->ppwrap.end)
 	{
@@ -393,8 +388,8 @@ wave_identify_offset (GslWaveChunk *wchunk,
 		g_print ("PHASE_PPWRAP %ld %ld %ld\n", wchunk->ppwrap.start, iter->rel_pos, wchunk->ppwrap.end);
 	      return PHASE_PPWRAP (wchunk);
 	    }
-	  iter->ubound = wchunk->loop_end - one - wchunk->wrap.end;
-	  iter->lbound = wchunk->loop_end - one - wchunk->ppwrap.start;
+	  iter->ubound = wchunk->loop_last - one - wchunk->wrap.end;
+	  iter->lbound = wchunk->loop_last - one - wchunk->ppwrap.start;
 	  iter->rel_pos = pos - wchunk->wrap.end;
 	  if (PRINT_DEBUG_INFO)
 	    g_print ("PHASE_NORM_BACKWARD, pploop %ld %ld %ld\n", iter->lbound, iter->rel_pos, iter->ubound);
@@ -408,17 +403,17 @@ wave_identify_offset (GslWaveChunk *wchunk,
 	  return PHASE_WRAP (wchunk);
 	}
       iter->rel_pos = pos - wchunk->ppwrap.end;
-      iter->ubound = wchunk->loop_start + one + wchunk->wrap.start - loop_width;
-      iter->lbound = wchunk->loop_start + one + wchunk->ppwrap.end - loop_width;
+      iter->ubound = wchunk->loop_first + one + wchunk->wrap.start - loop_width;
+      iter->lbound = wchunk->loop_first + one + wchunk->ppwrap.end - loop_width;
       if (PRINT_DEBUG_INFO)
 	g_print ("PHASE_NORM, pploop %ld %ld %ld\n", iter->lbound, iter->rel_pos, iter->ubound);
       return PHASE_NORM (wchunk);
     }
   else if (wchunk->loop_type == GSL_WAVE_LOOP_JUMP)		/* in jump loop */
     {
-      guint loop_width = wchunk->loop_end - wchunk->loop_start + one;
+      guint loop_width = wchunk->loop_last - wchunk->loop_first + one;
 
-      pos -= wchunk->loop_end + one;
+      pos -= wchunk->loop_last + one;
       pos %= loop_width;
       if (pos >= wchunk->wrap.start)
 	{
@@ -435,8 +430,8 @@ wave_identify_offset (GslWaveChunk *wchunk,
 	  return PHASE_WRAP (wchunk);
 	}
       iter->rel_pos = pos - wchunk->wrap.end;
-      iter->lbound = wchunk->loop_start + wchunk->wrap.end;
-      iter->ubound = wchunk->loop_start + wchunk->wrap.start;
+      iter->lbound = wchunk->loop_first + wchunk->wrap.end;
+      iter->ubound = wchunk->loop_first + wchunk->wrap.start;
       if (PRINT_DEBUG_INFO)
 	g_print ("PHASE_NORM, jloop %ld %ld %ld\n", iter->lbound, iter->rel_pos, iter->ubound);
       return PHASE_NORM (wchunk);
@@ -459,16 +454,20 @@ gsl_wave_chunk_use_block (GslWaveChunk      *wchunk,
   gboolean reverse;
 
   g_return_if_fail (wchunk != NULL);
+  g_return_if_fail (wchunk->open_count > 0);
   g_return_if_fail (block != NULL);
   g_return_if_fail (wchunk->dcache != NULL);
   g_return_if_fail (block->node == NULL);
   g_return_if_fail (block->play_dir == -1 || block->play_dir == +1);
 
+  block->offset /= wchunk->n_channels;
+  block->offset *= wchunk->n_channels;
+
   one = wchunk->n_channels;
   reverse = block->play_dir < 0;
   iter.pos = block->offset;
   phase = wave_identify_offset (wchunk, &iter);
-  
+
   block->is_silent = FALSE;
   if (phase <= PHASE_UNDEF (wchunk))
     {
@@ -486,7 +485,7 @@ gsl_wave_chunk_use_block (GslWaveChunk      *wchunk,
 	}
       else
 	{
-	  guint max_length;
+	  GslLong max_length;
 
 	  if (phase == PHASE_NORM_BACKWARD (wchunk))
 	    {
@@ -496,7 +495,6 @@ gsl_wave_chunk_use_block (GslWaveChunk      *wchunk,
 	  else
 	    offset = iter.lbound + iter.rel_pos;
 	  max_length = reverse ? offset - iter.lbound : iter.ubound - offset;
-	  offset += wchunk->offset;
 	  dnode = gsl_data_cache_ref_node (wchunk->dcache, offset, TRUE); /* FIXME: demand_load */
 	  offset -= dnode->offset;
 	  block->start = dnode->data + offset;
@@ -516,11 +514,6 @@ gsl_wave_chunk_use_block (GslWaveChunk      *wchunk,
     }
   else
     {
-      if (0)
-	g_print ("phase: head=%u enter=%u wrap=%u leave=%u tail=%u\n",
-		 phase == &wchunk->head, phase == &wchunk->enter,
-		 phase == &wchunk->wrap, phase == &wchunk->leave,
-		 phase == &wchunk->tail);
       block->start = phase->mem + iter.rel_pos;
       if (reverse)
 	block->length = one + iter.rel_pos;
@@ -559,75 +552,180 @@ gsl_wave_chunk_unuse_block (GslWaveChunk      *wchunk,
     }
 }
 
+static void
+wave_chunk_setup_loop (GslWaveChunk *wchunk)
+{
+  GslWaveLoopType loop_type = wchunk->requested_loop_type;
+  GslLong loop_first = wchunk->requested_loop_first;
+  GslLong loop_last = wchunk->requested_loop_last;
+  guint loop_count = wchunk->requested_loop_count;
+  GslLong one, padding, big_pad;
+
+  g_return_if_fail (wchunk->open_count > 0);
+
+  one = wchunk->n_channels;
+  padding = wchunk->n_pad_values;
+  big_pad = PBLOCK_SIZE (wchunk->n_pad_values, wchunk->n_channels);
+
+  /* check validity */
+  if (loop_count < 1 || loop_first < 0 || loop_last < 0 || wchunk->length < 1)
+    loop_type = GSL_WAVE_LOOP_NONE;
+
+  /* setup loop types */
+  switch (loop_type)
+    {
+    case GSL_WAVE_LOOP_JUMP:
+      loop_first /= wchunk->n_channels;
+      loop_last /= wchunk->n_channels;
+      if (loop_last >= wchunk->length ||
+	  loop_first >= loop_last)
+	goto CASE_DONT_LOOP;
+      wchunk->loop_type = loop_type;
+      wchunk->loop_first = loop_first * wchunk->n_channels;
+      wchunk->loop_last = loop_last * wchunk->n_channels;
+      wchunk->loop_count = (G_MAXINT - wchunk->length) / (wchunk->loop_last - wchunk->loop_first + one);
+      wchunk->loop_count = MIN (wchunk->loop_count, loop_count);
+      wchunk->wave_length = wchunk->length + (wchunk->loop_last - wchunk->loop_first + one) * wchunk->loop_count;
+      break;
+    case GSL_WAVE_LOOP_PINGPONG:
+      loop_first /= wchunk->n_channels;
+      loop_last /= wchunk->n_channels;
+      if (loop_last >= wchunk->length ||
+	  loop_first >= loop_last)
+	goto CASE_DONT_LOOP;
+      wchunk->loop_type = loop_type;
+      wchunk->loop_first = loop_first * wchunk->n_channels;
+      wchunk->loop_last = loop_last * wchunk->n_channels;
+      wchunk->loop_count = (G_MAXINT - wchunk->loop_last - one) / (wchunk->loop_last - wchunk->loop_first);
+      wchunk->loop_count = MIN (wchunk->loop_count, loop_count);
+      wchunk->wave_length = wchunk->loop_last + one + (wchunk->loop_last - wchunk->loop_first) * wchunk->loop_count;
+      if (wchunk->loop_count & 1)	/* FIXME */
+	wchunk->wave_length += wchunk->loop_first;
+      else
+	wchunk->wave_length += wchunk->length - one - wchunk->loop_last;
+      break;
+    CASE_DONT_LOOP:
+      loop_type = GSL_WAVE_LOOP_NONE;
+    case GSL_WAVE_LOOP_NONE:
+      wchunk->loop_type = loop_type;
+      wchunk->loop_first = wchunk->length + 1;
+      wchunk->loop_last = -1;
+      wchunk->loop_count = 0;
+      wchunk->wave_length = wchunk->length;
+      break;
+    }
+  wchunk->pploop_ends_backwards = wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG && (wchunk->loop_count & 1);
+  wchunk->mini_loop = wchunk->loop_type && wchunk->loop_last - wchunk->loop_first < 2 * big_pad + padding;
+}
+
 GslWaveChunk*
-_gsl_wave_chunk_create (GslDataCache   *dcache,
-			GslLong         offset,
-			GslLong         n_values, /* per channel */
-			guint           n_channels,
-			gfloat          osc_freq,
-			gfloat          mix_freq,
-			GslWaveLoopType loop_type,
-			GslLong         loop_start,
-			GslLong         loop_end,
-			guint           loop_count)
+gsl_wave_chunk_new (GslDataCache   *dcache,
+		    gfloat          osc_freq,
+		    gfloat          mix_freq,
+		    GslWaveLoopType loop_type,
+		    GslLong         loop_first,
+		    GslLong         loop_last,
+		    guint           loop_count)
 {
   GslWaveChunk *wchunk;
 
   g_return_val_if_fail (dcache != NULL, NULL);
-  g_return_val_if_fail (offset >= 0, NULL);
-  g_return_val_if_fail (n_values > 0, NULL);
-  g_return_val_if_fail (n_channels > 0, NULL);
-  g_return_val_if_fail (offset + n_values * n_channels <= dcache->handle->n_values, NULL);
   g_return_val_if_fail (osc_freq < mix_freq / 2, NULL);
-  g_return_val_if_fail (dcache->padding >= gsl_get_config ()->wave_chunk_padding * n_channels, NULL);
+  g_return_val_if_fail (loop_type >= GSL_WAVE_LOOP_NONE && loop_type <= GSL_WAVE_LOOP_PINGPONG, NULL);
 
   wchunk = gsl_new_struct0 (GslWaveChunk, 1);
   wchunk->dcache = gsl_data_cache_ref (dcache);
-  wchunk->owner_data = NULL;
-  wchunk->offset = offset;
-  wchunk->n_channels = n_channels;
-  wchunk->length = n_values * wchunk->n_channels;
-  wchunk->mix_freq = mix_freq;
-  wchunk->osc_freq = osc_freq;
-  wchunk->n_pad_values = gsl_get_config ()->wave_chunk_padding * n_channels;
-  wchunk->wave_length = wchunk->length;
-  wchunk->loop_type = 0;
-  wchunk->loop_start = wchunk->length + 1;
-  wchunk->loop_end = -1;
-  wchunk->loop_count = 0;
+  wchunk->length = 0;
+  wchunk->n_channels = 0;
+  wchunk->n_pad_values = 0;
+  wchunk->wave_length = 0;
+  wchunk->loop_type = GSL_WAVE_LOOP_NONE;
   wchunk->leave_end_norm = 0;
   wchunk->tail_start_norm = 0;
-  wave_chunk_set_loop (wchunk, loop_type, loop_start, loop_end, loop_count);
-  wave_chunk_open (wchunk);
+  wchunk->ref_count = 1;
+  wchunk->open_count = 0;
+  wchunk->mix_freq = mix_freq;
+  wchunk->osc_freq = osc_freq;
+  wchunk->requested_loop_type = loop_type;
+  wchunk->requested_loop_first = loop_first;
+  wchunk->requested_loop_last = loop_last;
+  wchunk->requested_loop_count = loop_count;
 
   return wchunk;
 }
 
+GslWaveChunk*
+gsl_wave_chunk_ref (GslWaveChunk *wchunk)
+{
+  g_return_val_if_fail (wchunk != NULL, NULL);
+  g_return_val_if_fail (wchunk->ref_count > 0, NULL);
+
+  wchunk->ref_count++;
+  return wchunk;
+}
+
 void
-_gsl_wave_chunk_destroy (GslWaveChunk *wchunk)
+gsl_wave_chunk_unref (GslWaveChunk *wchunk)
 {
   g_return_if_fail (wchunk != NULL);
+  g_return_if_fail (wchunk->ref_count > 0);
 
-  wave_chunk_close (wchunk);
-  gsl_data_cache_unref (wchunk->dcache);
-  gsl_delete_struct (GslWaveChunk, wchunk);
+  wchunk->ref_count--;
+  if (wchunk->ref_count == 0)
+    {
+      g_return_if_fail (wchunk->open_count == 0);
+      gsl_data_cache_unref (wchunk->dcache);
+      gsl_delete_struct (GslWaveChunk, wchunk);
+    }
 }
 
-static void
-wave_chunk_open (GslWaveChunk *wchunk)
+GslErrorType
+gsl_wave_chunk_open (GslWaveChunk *wchunk)
 {
-  g_return_if_fail (wchunk != NULL);
+  g_return_val_if_fail (wchunk != NULL, GSL_ERROR_INTERNAL);
+  g_return_val_if_fail (wchunk->ref_count > 0, GSL_ERROR_INTERNAL);
 
-  gsl_data_cache_open (wchunk->dcache);
-  setup_pblocks (wchunk);
+  if (wchunk->open_count == 0)
+    {
+      GslErrorType error;
+
+      error = gsl_data_handle_open (wchunk->dcache->dhandle);
+      if (error != GSL_ERROR_NONE)
+	return error;
+      if (gsl_data_handle_n_values (wchunk->dcache->dhandle) < gsl_data_handle_n_channels (wchunk->dcache->dhandle))
+	{
+	  gsl_data_handle_close (wchunk->dcache->dhandle);
+	  return GSL_ERROR_FILE_EMPTY;
+	}
+      wchunk->n_channels = gsl_data_handle_n_channels (wchunk->dcache->dhandle);
+      wchunk->length = gsl_data_handle_n_values (wchunk->dcache->dhandle) / wchunk->n_channels;
+      wchunk->length *= wchunk->n_channels;
+      wchunk->n_pad_values = gsl_get_config ()->wave_chunk_padding * wchunk->n_channels;
+      gsl_data_cache_open (wchunk->dcache);
+      gsl_data_handle_close (wchunk->dcache->dhandle);
+      g_return_val_if_fail (wchunk->dcache->padding >= wchunk->n_pad_values, GSL_ERROR_INTERNAL);
+      wchunk->open_count++;
+      wchunk->ref_count++;
+      wave_chunk_setup_loop (wchunk);
+      setup_pblocks (wchunk);
+    }
+  else
+    wchunk->open_count++;
+  return GSL_ERROR_NONE;
 }
 
-static void
-wave_chunk_close (GslWaveChunk *wchunk)
+void
+gsl_wave_chunk_close (GslWaveChunk *wchunk)
 {
   GslLong padding;
 
   g_return_if_fail (wchunk != NULL);
+  g_return_if_fail (wchunk->open_count > 0);
+  g_return_if_fail (wchunk->ref_count > 0);
+
+  wchunk->open_count--;
+  if (wchunk->open_count)
+    return;
 
   padding = wchunk->n_pad_values;
   gsl_data_cache_close (wchunk->dcache);
@@ -649,70 +747,14 @@ wave_chunk_close (GslWaveChunk *wchunk)
   if (wchunk->tail.mem)
     gsl_delete_structs (gfloat, wchunk->tail.length + 2 * padding, wchunk->tail.mem - padding);
   memset (&wchunk->tail, 0, sizeof (GslWaveChunkMem));
+  wchunk->length = 0;
+  wchunk->n_channels = 0;
+  wchunk->n_pad_values = 0;
+  wchunk->wave_length = 0;
+  wchunk->loop_type = GSL_WAVE_LOOP_NONE;
   wchunk->leave_end_norm = 0;
   wchunk->tail_start_norm = 0;
-}
-
-static void
-wave_chunk_set_loop (GslWaveChunk   *wchunk,
-		     GslWaveLoopType loop_type,
-		     GslLong         loop_start,
-		     GslLong         loop_end,
-		     guint           loop_count)
-{
-  GslLong one, padding, big_pad;
-
-  g_return_if_fail (wchunk != NULL);
-
-  if (loop_count < 1 || loop_start < 0 || loop_end < loop_start)
-    loop_type = GSL_WAVE_LOOP_NONE;
-  one = wchunk->n_channels;
-  padding = wchunk->n_pad_values;
-  big_pad = PBLOCK_SIZE (wchunk->n_pad_values, wchunk->n_channels);
-  switch (loop_type)
-    {
-    case GSL_WAVE_LOOP_NONE:
-      wchunk->loop_type = loop_type;
-      wchunk->loop_start = wchunk->length + 1;
-      wchunk->loop_end = -1;
-      wchunk->loop_count = 0;
-      wchunk->wave_length = wchunk->length;
-      break;
-    case GSL_WAVE_LOOP_JUMP:
-      g_return_if_fail (loop_start >= 0 && loop_start < wchunk->length);
-      g_return_if_fail (loop_end < wchunk->length);
-      loop_start /= wchunk->n_channels;
-      loop_end /= wchunk->n_channels;
-      g_return_if_fail (loop_start < loop_end);
-      wchunk->loop_type = loop_type;
-      wchunk->loop_start = loop_start * wchunk->n_channels;
-      wchunk->loop_end = loop_end * wchunk->n_channels;
-      wchunk->loop_count = (G_MAXINT - wchunk->length) / (wchunk->loop_end - wchunk->loop_start + one);
-      wchunk->loop_count = MIN (wchunk->loop_count, loop_count);
-      wchunk->wave_length = wchunk->length + (wchunk->loop_end - wchunk->loop_start + one) * wchunk->loop_count;
-      break;
-    case GSL_WAVE_LOOP_PINGPONG:
-      g_return_if_fail (loop_start >= 0 && loop_start < wchunk->length);
-      g_return_if_fail (loop_end < wchunk->length);
-      loop_start /= wchunk->n_channels;
-      loop_end /= wchunk->n_channels;
-      g_return_if_fail (loop_start < loop_end);
-      wchunk->loop_type = loop_type;
-      wchunk->loop_start = loop_start * wchunk->n_channels;
-      wchunk->loop_end = loop_end * wchunk->n_channels;
-      wchunk->loop_count = (G_MAXINT - wchunk->loop_end - one) / (wchunk->loop_end - wchunk->loop_start);
-      wchunk->loop_count = MIN (wchunk->loop_count, loop_count);
-      wchunk->wave_length = wchunk->loop_end + one + (wchunk->loop_end - wchunk->loop_start) * wchunk->loop_count;
-      if (wchunk->loop_count & 1)	/* FIXME */
-	wchunk->wave_length += wchunk->loop_start;
-      else
-	wchunk->wave_length += wchunk->length - one - wchunk->loop_end;
-      break;
-    default:
-      g_return_if_fail (loop_type >= GSL_WAVE_LOOP_NONE && loop_type <= GSL_WAVE_LOOP_PINGPONG);
-    }
-  wchunk->pploop_ends_backwards = wchunk->loop_type == GSL_WAVE_LOOP_PINGPONG && (wchunk->loop_count & 1);
-  wchunk->mini_loop = wchunk->loop_type && wchunk->loop_end - wchunk->loop_start < 2 * big_pad + padding;
+  gsl_wave_chunk_unref (wchunk);
 }
 
 void
@@ -727,20 +769,18 @@ gsl_wave_chunk_debug_block (GslWaveChunk *wchunk,
 }
 
 GslWaveChunk*
-gsl_wave_chunk_copy (GslWaveChunk *wchunk)
+_gsl_wave_chunk_copy (GslWaveChunk *wchunk)
 {
   g_return_val_if_fail (wchunk != NULL, NULL);
+  g_return_val_if_fail (wchunk->ref_count > 0, NULL);
 
-  return _gsl_wave_chunk_create (wchunk->dcache,
-				 wchunk->offset,
-				 wchunk->length / wchunk->n_channels,
-				 wchunk->n_channels,
-				 wchunk->osc_freq,
-				 wchunk->mix_freq,
-				 wchunk->loop_type,
-				 wchunk->loop_start,
-				 wchunk->loop_end,
-				 wchunk->loop_count);
+  return gsl_wave_chunk_new (wchunk->dcache,
+			     wchunk->osc_freq,
+			     wchunk->mix_freq,
+			     wchunk->loop_type,
+			     wchunk->loop_first,
+			     wchunk->loop_last,
+			     wchunk->loop_count);
 }
 
 const gchar*
