@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     Permission is also granted to link this program with the Qt
     library, treating Qt like a library that normally accompanies the
@@ -30,7 +30,7 @@
 #include <list>
 #include <stack>
 #include <ctype.h>
-#include "core.h"
+#include "common.h"
 #include "namespace.h"
 #include <iostream>
 
@@ -725,7 +725,7 @@ FILE *startHeader(string prefix)
 	string header_name = prefix+".h.new";
 	FILE *header = fopen(header_name.c_str(),"w");
 
-	fprintf(header,generated_disclaimer);
+	fprintf(header,"%s", generated_disclaimer);
 	fprintf(header,"#ifndef %s\n",mkdef(prefix).c_str());
 	fprintf(header,"#define %s\n\n",mkdef(prefix).c_str());
 	fprintf(header,"#include \"common.h\"\n\n");
@@ -750,7 +750,7 @@ FILE *startSource(string prefix)
 	string source_name = prefix+".cc.new";
 
 	FILE *source = fopen(source_name.c_str(),"w");
-	fprintf(source,generated_disclaimer);
+	fprintf(source,"%s", generated_disclaimer);
 	fprintf(source,"#include \"%s\"\n\n",header_name.c_str());
 
 	return source;
